import tempfile
import typing


def save_yaml_to_file(s: str, delete: bool = True) -> tempfile.NamedTemporaryFile:
    tempf = tempfile.NamedTemporaryFile(suffix=".yml", delete=delete)
    write_to_file(s, tempf.name)
    return tempf


def write_to_file(s: str, filename: str) -> None:
    with open(filename, "w") as f:
        f.write(s)


def merge_dicts(
    dict1: typing.Dict[typing.Any, typing.Any],
    dict2: typing.Dict[typing.Any, typing.Any],
) -> typing.Dict[typing.Any, typing.Any]:
    """ Recursively merges dict2 into dict1 """
    if not isinstance(dict1, dict) or not isinstance(dict2, dict):
        return dict1
    for k in dict2:
        if k in dict1:
            dict1[k] = merge_dicts(dict1[k], dict2[k])
        else:
            dict1[k] = dict2[k]
    return dict1



import unittest
from omegaconf import OmegaConf
import docker
import pytest

from ansible_please.ansible_runner import AnsibleRunner
from ansible_please.inventory import Inventory
from ansible_please.playbook import Playbook
from ansible_please.task_templates.docker_container import DockerContainer
from ansible_please.task_templates.pip import Pip


client = docker.DockerClient()

TEST_INPUT_DICT = dict(
    OmegaConf.create(
        """
hosts:
    master_host:
      - 'localhost'
    worker_hosts:
      - '127.0.0.1'
host_info:
  '127.0.0.1':
    'python_path': /usr/bin/python3
"""
    )
)


class TestAnsibleRunner(unittest.TestCase):

    def test_missing_inventory_creation(self):
        pip_task = Pip(packages=["docker"])
        playbook = Playbook(
            name="test-invalid-playbook",
            hosts="master_host",
            tasks=[
                pip_task.up(),
            ],
        )
        with pytest.raises(ValueError):
            runner = AnsibleRunner(inventory=None, playbook=playbook)

    def test_read_from_yaml(self):
        pip_task = Pip(packages=["docker"])
        playbook = Playbook(
            name="test-invalid-playbook",
            hosts="master_host",
            tasks=[
                pip_task.up(),
            ],
        )
        runner = AnsibleRunner(input_path="tests/integration/test_input.yml", playbook=playbook)
        assert True

    def test_run(self):
        inventory = Inventory(TEST_INPUT_DICT)
        pip_task = Pip(packages=["docker"])
        docker_container_task = DockerContainer(
            task_description="start-test-redis",
            name="test-redis",
            image="redis:latest",
        )
        playbook = Playbook(
            name="test-redis-playbook",
            hosts="master_host",
            tasks=[
                pip_task.up(),
                docker_container_task.up(),
                docker_container_task.down(),
            ],
        )
        runner = AnsibleRunner(inventory=inventory, playbook=playbook)

        runner.up()
        has_container = False
        for container in client.containers.list():
            if container.name == "test-redis":
                has_container = True
        assert has_container

        # tear down
        runner.down()
        has_container = False
        for container in client.containers.list():
            if container.name == "test-redis":
                has_container = True
        assert not has_container



from omegaconf import OmegaConf

EXPECTED_HOST_INPUT_DICT = dict(
    OmegaConf.create(
        """
hosts:
    master_host:
      - '127.0.0.1'
    worker_hosts:
      - '127.0.0.1'
host_info:
  '127.0.0.1':
"""
    )
)

EXPECTED_TEST_PLAYBOOK = dict(
    OmegaConf.create(
        """
- name: test_playbook
  hosts: test_hosts
  gather_facts: true
  tasks:
  - name: '[up] Install something with command line'
    command:
      cmd: echo test
    tags:
    - up
"""
    )[0]
)



import unittest
from unittest.mock import patch
from omegaconf import OmegaConf

from ansible_please.inventory import Inventory, get_ip  # noqa

TEST_INPUT_DICT = dict(
    OmegaConf.create(
        """
hosts:
    master_host:
      - 'localhost'
    worker_hosts:
      - '127.0.0.1'
host_info:
  '127.0.0.1':
    'python_path': /test_path/python3
"""
    )
)

EXPECTED_INVENTORY_DICT = {
    "all": {
        "children": {
            "master_host": {
                "hosts": {
                    "127.0.0.1": {
                        "ansible_connection": "local",
                        "host_checking": False,
                        "ansible_python_interpreter": "/test_path/python3",
                    }
                }
            },
            "worker_hosts": {
                "hosts": {
                    "127.0.0.1": {
                        "ansible_connection": "local",
                        "host_checking": False,
                        "ansible_python_interpreter": "/test_path/python3",
                    }
                }
            },
        },
        "vars": {
            "stdout_callback": "debug",
            "display_args_to_stdout": True,
            "host_key_checking": False,
            "look_for_keys": True,
        },
    }
}


class TestInventory(unittest.TestCase):
    @patch("ansible_please.inventory.get_ip", return_value="127.0.0.1")
    def test_inventory_creation(self, mock_get_ip):
        i = Inventory(TEST_INPUT_DICT)
        assert i.inventory_dict.data == EXPECTED_INVENTORY_DICT
        assert i.__str__()

    def test_get_ip(self):
        _ = get_ip()  # don't fail
        assert True

    @patch("ansible_please.inventory.get_sockname", side_effect=Exception('mocked error'))
    def test_failed_ip(self, mock_get_sockname):
        IP = get_ip()
        assert IP == "127.0.0.1"



import unittest
from ansible_please.ansible_task import AnsibleTask
from ansible_please.ansible_dict import AnsibleDict
import copy
import pytest

EXPECTED_UP_DICT = AnsibleDict(
    {
        "name": "[up] test description",
        "test_task_name": {"test": 2},
        "tags": ["test-tag", "up"],
    }
)

test_ansible_task = AnsibleTask(
    input_dict={"test": 2},
    task_name="test_task_name",
    task_description="test description",
    tags=["test-tag"],
)


class TestAnsibleTask(unittest.TestCase):
    def test_invalid_name(self):
        with pytest.raises(ValueError):
            _ = AnsibleTask(task_name="")

    def test__str__(self):
        assert test_ansible_task.__str__() == EXPECTED_UP_DICT.__str__()

    def test_asdict(self):
        actual = test_ansible_task.asdict()
        assert actual == EXPECTED_UP_DICT

    def test_up(self):
        actual = test_ansible_task.up().asdict()
        assert actual == EXPECTED_UP_DICT

    def test_down(self):
        actual = test_ansible_task.down().asdict()
        expected = copy.deepcopy(EXPECTED_UP_DICT.data)
        expected["name"] = "[down] test description"
        expected["tags"] = ["test-tag", "down"]
        assert actual == AnsibleDict(expected)



import unittest
from ansible_please.base_ansible import BaseAnsible
from ansible_please.ansible_dict import AnsibleDict
from unittest.mock import patch


class TestBaseAnsibleClass(BaseAnsible):
    def __init__(self, input_dict={"test_dict": {"z": "testz"}}):
        super(TestBaseAnsibleClass, self).__init__(input_dict)
        self.x = "testx"
        self.y = "testy"
        self.zlist = [1, 2]
        self.base_ansible = BaseAnsible(input_dict={"test_input": "test"})
        self.ansible_dict = AnsibleDict(data={"test_data": "test"})


EXPECTED_DICT: AnsibleDict = AnsibleDict(
    {
        "x": "testx",
        "y": "testy",
        "zlist": [1, 2],
        "base_ansible": {"test_input": "test"},
        "ansible_dict": {"test_data": "test"},
        "test_dict": {"z": "testz"},
    }
)


class TestBaseAnsible(unittest.TestCase):
    def test_as_dict(self):
        actual = TestBaseAnsibleClass().asdict()
        assert actual == EXPECTED_DICT

    def test__str__(self):
        actual = TestBaseAnsibleClass().__str__()
        print("ACTUAL", actual)
        print("-------------------")
        print("EXPECTED_DICT", EXPECTED_DICT)
        assert actual == EXPECTED_DICT.__str__()

    @patch("ansible_please.base_ansible.save_yaml_to_file", return_value=None)
    @patch("ansible_please.ansible_dict.save_yaml_to_file", return_value=None)
    def test_save_to_file(
        self, mock_save_yaml_to_file, mock_ansible_dict_save_yaml_to_file
    ):
        """should be covered in test_utils
        """
        _ = TestBaseAnsibleClass().save_to_file()
        assert True
        _ = AnsibleDict({"test": None}).save_to_file()
        assert True
